@@ -1,4 +1,4 @@
-/*	$calcurse: args.c,v 1.55 2009-07-12 18:16:11 culot Exp $	*/
+/*	$calcurse: args.c,v 1.59 2009-08-02 07:22:50 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -40,6 +40,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <sys/types.h>
+#include <limits.h>
 #include <getopt.h>
 #include <time.h>
 #include <regex.h>
@@ -68,7 +69,7 @@ usage ()
   char *arg_usage =
     _("Usage: calcurse [-h|-v] [-N] [-an] [-t[num]] [-i<file>] [-x[format]]\n"
       "                [-d <date>|<num>] [-s[date]] [-r[range]]\n"
-      "                [-c<file> | -D<dir>] [-S<regex>]\n");
+      "                [-c<file> | -D<dir>] [-S<regex>] [--status]\n");
   fputs (arg_usage, stdout);
 }
 
@@ -109,6 +110,8 @@ help_arg ()
       "	print this help and exit.\n"
       "\n  -v, --version\n"
       "	print calcurse version and exit.\n"
+      "\n  --status\n"
+      "	display the status of running instances of calcurse.\n"
       "\nFiles:\n"
       "  -c <file>, --calendar <file>\n"
       "	specify the calendar <file> to use (incompatible with '-D').\n"
@@ -165,6 +168,39 @@ help_arg ()
   fputs (htext, stdout);
 }
 
+/*
+ * Used to display the status of running instances of calcurse.
+ * The displayed message will look like one of the following ones:
+ *
+ *   calcurse is running (pid #)
+ *   calcurse is running in background (pid #)
+ *   calcurse is not running
+ *
+ * The status is obtained by looking at pid files in user data directory
+ * (.calcurse.pid and .daemon.pid).
+ */
+static void
+status_arg (void)
+{
+  int cpid, dpid;
+  
+  cpid = io_get_pid (path_cpid);
+  dpid = io_get_pid (path_dpid);
+
+  EXIT_IF (cpid && dpid,
+           _("Error: both calcurse (pid: %d) and its daemon (pid: %d)\n"
+             "seem to be running at the same time!\n"
+             "Please check manually and restart calcurse.\n"),
+           cpid, dpid);
+
+  if (cpid)
+    fprintf (stdout, _("calcurse is running (pid %d)\n"), cpid);
+  else if (dpid)
+    fprintf (stdout, _("calcurse is running in background (pid %d)\n"), dpid);
+  else
+    fprintf (stdout, _("calcurse is not running\n"));
+}
+
 /*
  * Display note contents if one is asociated with the currently displayed item
  * (to be used together with the '-a' or '-t' flag in non-interactive mode).
@@ -409,6 +445,8 @@ app_arg (int add_line, date_t *day, long date, int print_note, conf_t *conf,
       if (recur_item_inday (ra->start, ra->exc, ra->rpt->type, ra->rpt->freq,
                             ra->rpt->until, today))
 	{
+          apoint_llist_node_t *apt;
+          
           if (regex && regexec (regex, ra->mesg, 0, 0, 0) != 0)
             continue;
           
@@ -423,8 +461,11 @@ app_arg (int add_line, date_t *day, long date, int print_note, conf_t *conf,
 	      arg_print_date (today, conf);
 	      print_date = 0;
 	    }
-	  apoint_sec2str (apoint_recur_s2apoint_s (ra), RECUR_APPT, today,
-                          apoint_start_time, apoint_end_time);
+          apt = apoint_recur_s2apoint_s (ra);
+	  apoint_sec2str (apt, RECUR_APPT, today, apoint_start_time,
+                          apoint_end_time);
+          mem_free (apt->mesg);
+          mem_free (apt);
 	  fputs (" - ", stdout);
 	  fputs (apoint_start_time, stdout);
 	  fputs (" -> ", stdout);
@@ -655,12 +696,19 @@ parse_args (int argc, char **argv, conf_t *conf)
   int tflag = 0;    /* -t: print todo list */
   int vflag = 0;    /* -v: print version number */
   int xflag = 0;    /* -x: export data */
-
+  
   int tnum = 0, xfmt = 0, non_interactive = 0, multiple_flag = 0, load_data = 0;
   char *ddate = "", *cfile = NULL, *range = NULL, *startday = NULL;
   char *datadir = NULL, *ifile = NULL;
   regex_t reg, *preg = NULL;
 
+  /* Long options only */
+  int statusflag = 0; /* --status: get the status of running instances */
+  enum
+  {
+    STATUS_OPT = CHAR_MAX + 1 
+  };
+  
   static char *optstr = "hvnNax::t::d:c:r::s::S:D:i:";
 
   struct option longopts[] = {
@@ -675,6 +723,7 @@ parse_args (int argc, char **argv, conf_t *conf)
     {"range", optional_argument, NULL, 'r'},
     {"startday", optional_argument, NULL, 's'},
     {"search", required_argument, NULL, 'S'},
+    {"status", no_argument, NULL, STATUS_OPT},
     {"todo", optional_argument, NULL, 't'},
     {"version", no_argument, NULL, 'v'},
     {"export", optional_argument, NULL, 'x'},
@@ -685,6 +734,9 @@ parse_args (int argc, char **argv, conf_t *conf)
     {
       switch (ch)
 	{
+        case STATUS_OPT:
+          statusflag = 1;
+          break;
 	case 'a':
 	  aflag = 1;
 	  multiple_flag++;
@@ -838,6 +890,12 @@ parse_args (int argc, char **argv, conf_t *conf)
 	  version_arg ();
 	  non_interactive = 1;
 	}
+      else if (statusflag)
+        {
+          io_init (cfile, datadir);          
+          status_arg ();
+          non_interactive = 1;
+        }
       else if (multiple_flag)
 	{
 	  if (load_data)
@@ -867,19 +925,24 @@ parse_args (int argc, char **argv, conf_t *conf)
 	    }
 	  if (tflag)
 	    {
-              io_load_todo ();              
+              io_check_file (path_todo, (int *)0);
+              io_load_todo ();
 	      todo_arg (tnum, Nflag, preg);
 	      non_interactive = 1;
 	    }
 	  if (nflag)
 	    {
+              io_check_file (path_apts, (int *)0);              
               io_load_app ();
 	      next_arg ();
 	      non_interactive = 1;
 	    }
 	  if (dflag || rflag || sflag)
 	    {
+              io_check_file (path_apts, (int *)0);
+              io_check_file (path_conf, (int *)0);              
               io_load_app ();
+              custom_load_conf (conf, 0); /* To get output date format. */
               if (dflag)
                 date_arg (ddate, add_line, Nflag, conf, preg);
               if (rflag || sflag)
@@ -891,6 +954,8 @@ parse_args (int argc, char **argv, conf_t *conf)
 	    {
 	      date_t day;
 
+              io_check_file (path_apts, (int *)0);              
+              io_check_file (path_conf, (int *)0);              
               vars_init (conf);
               custom_load_conf (conf, 0); /* To get output date format. */
               io_load_app ();
