@@ -1,4 +1,4 @@
-/*	$calcurse: custom.c,v 1.41 2009-07-12 17:55:13 culot Exp $	*/
+/*	$calcurse: custom.c,v 1.44 2009-08-02 09:29:24 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -222,7 +222,7 @@ custom_load_conf (conf_t *conf, int background)
   FILE *data_file;
   char *mesg_line1 = _("Failed to open config file");
   char *mesg_line2 = _("Press [ENTER] to continue");
-  char buf[100], e_conf[100];
+  char buf[BUFSIZ], e_conf[BUFSIZ];
   int var;
 
   data_file = fopen (path_conf, "r");
@@ -237,11 +237,11 @@ custom_load_conf (conf_t *conf, int background)
   pthread_mutex_lock (&nbar.mutex);
   for (;;)
     {
-      if (fgets (buf, 99, data_file) == NULL)
+      if (fgets (buf, sizeof buf, data_file) == NULL)
 	{
 	  break;
 	}
-      io_extract_data (e_conf, buf, strlen (buf));
+      io_extract_data (e_conf, buf, sizeof buf);
 
       switch (var)
 	{
@@ -319,6 +319,14 @@ custom_load_conf (conf_t *conf, int background)
 	    conf->input_datefmt = 1;
 	  var = 0;
 	  break;
+	case CUSTOM_CONF_DMON_ENABLE:
+	  dmon.enable = fill_config_var (e_conf);
+	  var = 0;
+	  break;
+	case CUSTOM_CONF_DMON_LOG:
+	  dmon.log = fill_config_var (e_conf);
+	  var = 0;
+	  break;
 	default:
           EXIT (_("configuration variable unknown"));
 	  /* NOTREACHED */
@@ -352,10 +360,14 @@ custom_load_conf (conf_t *conf, int background)
 	var = CUSTOM_CONF_NOTIFYBARWARNING;
       else if (strncmp (e_conf, "notify-bar_command=", 19) == 0)
 	var = CUSTOM_CONF_NOTIFYBARCOMMAND;
-      else if (strncmp (e_conf, "output_datefmt=", 12) == 0)
+      else if (strncmp (e_conf, "output_datefmt=", 15) == 0)
 	var = CUSTOM_CONF_OUTPUTDATEFMT;
-      else if (strncmp (e_conf, "input_datefmt=", 12) == 0)
+      else if (strncmp (e_conf, "input_datefmt=", 14) == 0)
 	var = CUSTOM_CONF_INPUTDATEFMT;
+      else if (strncmp (e_conf, "notify-daemon_enable=", 21) == 0)
+        var = CUSTOM_CONF_DMON_ENABLE;
+      else if (strncmp (e_conf, "notify-daemon_log=", 18) == 0)
+        var = CUSTOM_CONF_DMON_LOG;
     }
   file_close (data_file, __FILE_POS__);
   pthread_mutex_unlock (&nbar.mutex);
@@ -967,8 +979,8 @@ print_general_options (WINDOW *win, conf_t *conf)
   return y + YOFF;
 }
 
-static void
-conf_set_scrsize (scrollwin_t *sw)
+void
+custom_set_swsiz (scrollwin_t *sw)
 {
   sw->win.x = 0;
   sw->win.y = 0;
@@ -1001,7 +1013,7 @@ custom_general_config (conf_t *conf)
   char *buf;
 
   clear ();
-  conf_set_scrsize (&cwin);
+  custom_set_swsiz (&cwin);
   (void)snprintf (cwin.label, BUFSIZ, _("general options"));
   wins_scrollwin_init (&cwin);
   wins_show (cwin.win.p, cwin.label);
@@ -1021,7 +1033,7 @@ custom_general_config (conf_t *conf)
           wins_reset ();
 	  wins_scrollwin_delete (&cwin);
 	  wins_scrollwin_init (&cwin);
-          conf_set_scrsize (&cwin);
+          custom_set_swsiz (&cwin);
           wins_show (cwin.win.p, cwin.label);
 	  cwin.first_visible_line = 0;          
 	  delwin (win[STA].p);
@@ -1199,7 +1211,7 @@ custom_keys_config (void)
   const int LABELLINES = 3;
   
   clear ();
-  conf_set_scrsize (&kwin);
+  custom_set_swsiz (&kwin);
   nbdisplayed = (kwin.win.h - LABELLINES) / LINESPERKEY;
   (void)snprintf (kwin.label, BUFSIZ, _("keys configuration"));
   wins_scrollwin_init (&kwin);
