@@ -1,4 +1,4 @@
-/*	$calcurse: io.c,v 1.68 2009-07-12 17:48:13 culot Exp $	*/
+/*	$calcurse: io.c,v 1.78 2009-08-09 16:00:02 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -39,6 +39,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <sys/types.h>
 #include <time.h>
 #include <math.h>
 #include <unistd.h>
@@ -514,7 +515,7 @@ ical_export_recur_apoints (FILE *stream)
       date_sec2date_fmt (i->start, ICALDATETIMEFMT, ical_datetime);
       (void)fprintf (stream, "BEGIN:VEVENT\n");
       (void)fprintf (stream, "DTSTART:%s\n", ical_datetime);
-      (void)fprintf (stream, "DURATION:PT%ldS\n", i->dur);
+      (void)fprintf (stream, "DURATION:PT0H0M%ldS\n", i->dur);
       (void)fprintf (stream, "RRULE:FREQ=%s;INTERVAL=%d",
                      ical_recur_type[i->rpt->type], i->rpt->freq);
 
@@ -617,7 +618,7 @@ ical_export_apoints (FILE *stream)
       date_sec2date_fmt (i->start, ICALDATETIMEFMT, ical_datetime);
       (void)fprintf (stream, "BEGIN:VEVENT\n");
       (void)fprintf (stream, "DTSTART:%s\n", ical_datetime);
-      (void)fprintf (stream, "DURATION:PT%ldS\n", i->dur);
+      (void)fprintf (stream, "DURATION:PT0H0M%ldS\n", i->dur);
       (void)fprintf (stream, "SUMMARY:%s\n", i->mesg);
       if (i->state & APOINT_NOTIFY)
 	ical_export_valarm (stream);
@@ -674,6 +675,34 @@ pcal_export_todo (FILE *stream)
   (void)fprintf (stream, "\n");
 }
 
+/* Append a line to a file. */
+unsigned
+io_fprintln (const char *fname, const char *fmt, ...)
+{
+  FILE *fp;
+  va_list ap;
+  char buf[BUFSIZ];
+  int ret;
+
+  fp = fopen (fname, "a");
+  RETVAL_IF (!fp, 0, _("Failed to open \"%s\", - %s\n"),
+             fname, strerror (errno));
+
+  va_start (ap, fmt);
+  ret = vsnprintf (buf, sizeof buf, fmt, ap);
+  RETVAL_IF (ret < 0, 0, _("Failed to build message\n"));
+  va_end (ap);
+  
+  ret = fprintf (fp, "%s", buf);
+  RETVAL_IF (ret < 0, 0, _("Failed to print message \"%s\"\n"), buf);
+  
+  ret = fclose (fp);
+  RETVAL_IF (ret != 0, 0, _("Failed to close \"%s\" - %s\n"),
+             fname, strerror (errno));
+
+  return 1;
+}
+
 /* 
  * Initialization of data paths. The cfile argument is the variable
  * which contains the calendar file. If none is given, then the default
@@ -698,7 +727,9 @@ io_init (char *cfile, char *datadir)
       (void)snprintf (path_notes, BUFSIZ, "%s/" NOTES_DIR_NAME, home);
       (void)snprintf (path_apts, BUFSIZ, "%s/" APTS_PATH_NAME, home);
       (void)snprintf (path_keys, BUFSIZ, "%s/" KEYS_PATH_NAME, home);
-      (void)snprintf (path_lock, BUFSIZ, "%s/" LOCK_PATH_NAME, home);
+      (void)snprintf (path_cpid, BUFSIZ, "%s/" CPID_PATH_NAME, home);
+      (void)snprintf (path_dpid, BUFSIZ, "%s/" DPID_PATH_NAME, home);      
+      (void)snprintf (path_dmon_log, BUFSIZ, "%s/" DLOG_PATH_NAME, home);
     }
   else
     {
@@ -711,7 +742,9 @@ io_init (char *cfile, char *datadir)
       (void)snprintf (path_todo, BUFSIZ, "%s/" TODO_PATH, home);
       (void)snprintf (path_conf, BUFSIZ, "%s/" CONF_PATH, home);
       (void)snprintf (path_keys, BUFSIZ, "%s/" KEYS_PATH, home);
-      (void)snprintf (path_lock, BUFSIZ, "%s/" LOCK_PATH, home);      
+      (void)snprintf (path_cpid, BUFSIZ, "%s/" CPID_PATH, home);
+      (void)snprintf (path_dpid, BUFSIZ, "%s/" DPID_PATH, home);      
+      (void)snprintf (path_dmon_log, BUFSIZ, "%s/" DLOG_PATH, home);           
       (void)snprintf (path_notes, BUFSIZ, "%s/" NOTES_DIR, home);
       if (cfile == NULL)
         {
@@ -762,11 +795,12 @@ io_init (char *cfile, char *datadir)
     }
 }
 
-  /* get data from file */
 void
 io_extract_data (char *dst_data, const char *org, int len)
 {
-  for (;;)
+  int i;
+  
+  for (i = 0; i < len - 1; i++)
     {
       if (*org == '\n' || *org == '\0')
 	break;
@@ -906,7 +940,18 @@ io_save_conf (conf_t *conf)
   
   if (ui_mode == UI_CURSES)
     pthread_mutex_unlock (&nbar.mutex);
-  
+
+  (void)fprintf (fp, "\n# If this option is set to yes, "
+                 "calcurse will run in background to get notifications "
+                 "after exiting\n");
+  (void)fprintf (fp, "notify-daemon_enable=\n");
+  (void)fprintf (fp, "%s\n", dmon.enable ? "yes" : "no");
+
+    (void)fprintf (fp, "\n# If this option is set to yes, "
+                   "activity will be logged when running in background\n");
+  (void)fprintf (fp, "notify-daemon_log=\n");
+  (void)fprintf (fp, "%s\n", dmon.log ? "yes" : "no");
+    
   file_close (fp, __FILE_POS__);
 
   return 1;
@@ -1263,11 +1308,11 @@ io_load_todo (void)
       else
 	note[0] = '\0';
       /* Then read todo description. */
-      (void)fgets (buf, BUFSIZ, data_file);
+      (void)fgets (buf, sizeof buf, data_file);
       newline = strchr (buf, '\n');
       if (newline)
         *newline = '\0';
-      io_extract_data (e_todo, buf, strlen (buf));
+      io_extract_data (e_todo, buf, sizeof buf);
       todo_add (e_todo, id, note);
       ++nb_tod;
     }
@@ -1466,14 +1511,30 @@ check_directory (char *dir, int *missing)
     (*missing)++;
 }
 
+unsigned
+io_file_exist (char *file)
+{
+  FILE *fd;
+  
+  if (!file)
+    return 0;
+
+  if ((fd = fopen (file, "r")) == 0)
+    return 0;
+
+  (void)fclose (fd);
+
+  return 1;
+}
+
 void
 io_check_file (char *file, int *missing)
 {
-  FILE *fd;
-
   errno = 0;
-  if ((fd = fopen (file, "r")) == NULL)
+  if (!io_file_exist (file))
     {
+      FILE *fd;
+      
       if (missing)
         (*missing)++;
       if ((fd = fopen (file, "w")) == NULL)
@@ -1482,8 +1543,8 @@ io_check_file (char *file, int *missing)
                          file, strerror (errno));
 	  exit_calcurse (EXIT_FAILURE);
 	}
+      file_close (fd, __FILE_POS__);
     }
-  file_close (fd, __FILE_POS__);
 }
 
 /* 
@@ -1666,7 +1727,7 @@ ical_store_todo (int priority, char *mesg, char *note)
 {
   todo_add (mesg, priority, note);
   mem_free (mesg);
-  mem_free (note);
+  erase_note (&note, ERASE_FORCE_KEEP_NOTE);
 }
 
 static void
@@ -1698,7 +1759,7 @@ ical_store_event (char *mesg, char *note, long day, long end, ical_rpt_t *rpt,
       event_new (mesg, note, day, EVENTID);
     }
   mem_free (mesg);
-  mem_free (note);
+  erase_note (&note, ERASE_FORCE_KEEP_NOTE);
 }
 
 static void
@@ -1709,7 +1770,7 @@ ical_store_apoint (char *mesg, char *note, long start, long dur,
 
   if (has_alarm)
     state |= APOINT_NOTIFY;
-  if (rpt != NULL)
+  if (rpt)
     {
       recur_apoint_new (mesg, note, start, dur, state, rpt->type, rpt->freq,
                         rpt->until, &exc);
@@ -1720,7 +1781,7 @@ ical_store_apoint (char *mesg, char *note, long start, long dur,
       apoint_new (mesg, note, start, dur, state);
     }
   mem_free (mesg);
-  mem_free (note);
+  erase_note (&note, ERASE_FORCE_KEEP_NOTE);
 }
 
 /*
@@ -2305,6 +2366,7 @@ ical_read_note (char *first_line, FILE *fdi, unsigned *noskipped,
         {
           file_close (fdo, __FILE_POS__);
           erase_note (&notename, ERASE_FORCE);
+          mem_free (notestr);
           return NULL;
         }
       else
@@ -2522,11 +2584,15 @@ ical_read_event (FILE *fdi, FILE *log, unsigned *noevents, unsigned *noapoints,
             "The end of item was not found."));
 
 cleanup:
-  
-  mem_free (vevent.note);
-  mem_free (vevent.mesg);
-  mem_free (vevent.rpt);
-  mem_free (vevent.exc);
+
+  if (vevent.note)
+    mem_free (vevent.note);
+  if (vevent.mesg)
+    mem_free (vevent.mesg);
+  if (vevent.rpt)
+    mem_free (vevent.rpt);
+  if (vevent.exc)
+    mem_free (vevent.exc);
   (*noskipped)++;
 }
 
@@ -2619,9 +2685,11 @@ ical_read_todo (FILE *fdi, FILE *log, unsigned *notodos, unsigned *noskipped,
             "The end of item was not found."));
 
 cleanup:
-  
-  mem_free (vtodo.note);
-  mem_free (vtodo.mesg);
+
+  if (vtodo.note)
+    mem_free (vtodo.note);
+  if (vtodo.mesg)
+    mem_free (vtodo.mesg);
   (*noskipped)++;
 }
 
@@ -2898,32 +2966,67 @@ io_set_lock (void)
 {
   FILE *lock;
 
-  if ((lock = fopen (path_lock, "r")) != NULL)
+  if ((lock = fopen (path_cpid, "r")) != NULL)
     {
       (void)fprintf (stderr,
                      _("\nWARNING: it seems that another calcurse instance is "
                        "already running.\n"
                        "If this is not the case, please remove the following "
                        "lock file: \n\"%s\"\n"
-                       "and restart calcurse.\n"), path_lock);
+                       "and restart calcurse.\n"), path_cpid);
       exit (EXIT_FAILURE);
     }
   else
     {
-      if ((lock = fopen (path_lock, "w")) == NULL)
-	{
-	  (void)fprintf (stderr, _("FATAL ERROR: could not create %s: %s\n"),
-                         path_lock, strerror (errno));
-	  exit_calcurse (EXIT_FAILURE);
-	}
-      (void)fclose (lock);
+      if (!io_dump_pid (path_cpid))
+        EXIT (_("FATAL ERROR: could not create %s: %s\n"),
+                         path_cpid, strerror (errno));
     }
 }  
 
-/* Used when calcurse exits to remove the lock file. */
-void
-io_unset_lock (void)
+/*
+ * Create a new file and write the process pid inside
+ * (used to create a simple lock for example).
+ */
+unsigned
+io_dump_pid (char *file)
 {
-  if (unlink (path_lock) != 0)
-    EXIT (_("Could not remove lock file: %s\n"), strerror (errno));
+  pid_t pid;
+  FILE *fp;
+  
+  if (!file)
+    return 0;
+
+  pid = getpid ();
+  if (!(fp = fopen (file, "w"))
+      || fprintf (fp, "%ld\n", (long)pid) < 0
+      || fclose (fp) != 0)
+    return 0;
+  
+  return 1;
+}
+
+/*
+ * Return the pid number contained in a file previously created with
+ * io_dump_pid ().
+ * If no file was found, return 0.
+ */
+unsigned
+io_get_pid (char *file)
+{
+  FILE *fp;
+  unsigned pid;
+
+  if (!file)
+    return 0;
+
+  if ((fp = fopen (file, "r")) == 0)
+    return 0;
+
+  if (fscanf (fp, "%u", &pid) != 1)
+    return 0;
+
+  (void)fclose (fp);
+
+  return pid;
 }
