@@ -1,4 +1,4 @@
-/*	$calcurse: io.h,v 1.22 2009-07-12 16:22:01 culot Exp $	*/
+/*	$calcurse: io.h,v 1.26 2009-07-27 21:00:41 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -63,6 +63,7 @@ typedef struct {
   char name[BUFSIZ];
 } io_file_t;
 
+unsigned    io_fprintln (const char *, const char *, ...);
 void        io_init (char *, char *);
 void        io_extract_data (char *, const char *, int);
 unsigned    io_save_conf (conf_t *);
@@ -73,6 +74,7 @@ void        io_save_cal (conf_t *, io_save_display_t);
 void        io_load_app (void);
 void        io_load_todo (void);
 void        io_load_keys (char *);
+unsigned    io_file_exist (char *);
 void        io_check_file (char *, int *);
 int         io_check_data_files (void);
 void        io_startup_screen (unsigned, int);
@@ -86,6 +88,7 @@ void        io_log_free (io_file_t *);
 void        io_start_psave_thread (conf_t *);
 void        io_stop_psave_thread (void);
 void        io_set_lock (void);
-void        io_unset_lock (void);
+unsigned    io_dump_pid (char *);
+unsigned    io_get_pid (char *);
 
 #endif /* !CALCURSE_IO_H */
