@@ -1,4 +1,4 @@
-/*	$calcurse: notify.c,v 1.40 2009-07-12 17:55:14 culot Exp $	*/
+/*	$calcurse: notify.c,v 1.47 2009-08-17 10:04:39 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -57,12 +57,44 @@ static struct notify_app_s    notify_app;
 static pthread_attr_t         detached_thread_attr;
 static pthread_t              notify_t_main;
 
+/*
+ * Return the number of seconds before next appointment
+ * (0 if no upcoming appointment).
+ */
+int
+notify_time_left (void)
+{
+  struct tm *ntime;
+  time_t ntimer;
+  int left;  
+
+  ntimer = time (NULL);
+  ntime = localtime (&ntimer);
+  left = notify_app.time - ntimer;
+
+  return left > 0 ? left : 0;
+}
+
+/*
+ * Return 1 if the reminder was not sent already for the upcoming
+ * appointment.
+ */
+unsigned
+notify_needs_reminder (void)
+{
+  if (notify_app.got_app
+      && (notify_app.state & APOINT_NOTIFY)
+      && !(notify_app.state & APOINT_NOTIFIED))
+    return 1;
+  return 0;
+}
+
 /*
  * This is used to update the notify_app structure.
  * Note: the mutex associated with this structure must be locked by the
  * caller!
  */
-static void
+void
 notify_update_app (long start, char state, char *msg)
 {
   notify_free_app ();
@@ -146,8 +178,12 @@ notify_init_bar (void)
 void
 notify_free_app (void)
 {
+  notify_app.time = 0;
+  notify_app.got_app = 0;
+  notify_app.state = APOINT_NULL;
   if (notify_app.txt)
     mem_free (notify_app.txt);
+  notify_app.txt = 0;
 }
 
 /* Stop the notify-bar main thread. */
@@ -170,18 +206,35 @@ notify_reinit_bar (void)
 }
 
 /* Launch user defined command as a notification. */
-static void
-launch_cmd (char *cmd, char *shell)
+unsigned
+notify_launch_cmd (void)
 {
   int pid;
 
+  if (notify_app.state & APOINT_NOTIFIED)
+    return 1;
+  
+  notify_app.state |= APOINT_NOTIFIED;
+    
   pid = fork ();
 
   if (pid < 0)
-    ERROR_MSG (_("error while launching command: could not fork"));
-  else if (pid == 0)		/* Child: launch user defined command */
-    if (execlp (shell, shell, "-c", cmd, (char *)0) < 0)
-      ERROR_MSG (_("error while launching command"));
+    {
+      ERROR_MSG (_("error while launching command: could not fork"));
+      return 0;
+    }
+  else if (pid == 0)
+    {
+      /* Child: launch user defined command */
+      if (execlp (nbar.shell, nbar.shell, "-c", nbar.cmd, (char *)0) < 0)
+        {
+          ERROR_MSG (_("error while launching command"));
+          _exit (1);
+        }
+      _exit (0);
+    }
+
+  return 1;
 }
 
 /* 
@@ -193,8 +246,7 @@ notify_update_bar (void)
 {
   const int space = 3;
   int file_pos, date_pos, app_pos, txt_max_len, too_long = 0;
-  int time_left, hours_left, minutes_left;
-  int blinking;
+  int time_left, blinking;
   char buf[BUFSIZ];
 
   date_pos = space;
@@ -220,9 +272,11 @@ notify_update_bar (void)
 	  (void)strncpy (buf, notify_app.txt, txt_max_len - 3);
 	  buf[txt_max_len - 3] = '\0';
 	}
-      time_left = notify_app.time - notify.time_in_sec;
+      time_left = notify_time_left ();
       if (time_left > 0)
 	{
+          int hours_left, minutes_left;
+          
 	  hours_left = (time_left / HOURINSEC);
 	  minutes_left = (time_left - hours_left * HOURINSEC) / MININSEC;
 	  pthread_mutex_lock (&nbar.mutex);
@@ -243,11 +297,8 @@ notify_update_bar (void)
 	  if (blinking)
 	    wattroff (notify.win, A_BLINK);
 
-	  if (blinking && !(notify_app.state & APOINT_NOTIFIED))
-	    {
-	      notify_app.state |= APOINT_NOTIFIED;
-	      launch_cmd (nbar.cmd, nbar.shell);
-	    }
+	  if (blinking)
+            (void)notify_launch_cmd ();
 	  pthread_mutex_unlock (&nbar.mutex);
 	}
       else
@@ -287,14 +338,13 @@ notify_main_thread (void *arg)
       ntimer = time (NULL);
       ntime = localtime (&ntimer);
       pthread_mutex_lock (&notify.mutex);
-      notify.time_in_sec = ntimer;
       pthread_mutex_lock (&nbar.mutex);
       strftime (notify.time, NOTIFY_FIELD_LENGTH, nbar.timefmt, ntime);
       strftime (notify.date, NOTIFY_FIELD_LENGTH, nbar.datefmt, ntime);
       pthread_mutex_unlock (&nbar.mutex);
       pthread_mutex_unlock (&notify.mutex);
       notify_update_bar ();
-      (void)sleep (thread_sleep);
+      psleep (thread_sleep);
       elapse += thread_sleep;
       if (elapse >= check_app)
 	{
@@ -309,38 +359,95 @@ notify_main_thread (void *arg)
   pthread_exit ((void *) 0);
 }
 
+/* Fill the given structure with information about next appointment. */
+unsigned
+notify_get_next (struct notify_app_s *a)
+{
+  time_t current_time;
+  
+  if (!a)
+    return 0;
+  
+  current_time = time (NULL);
+
+  a->time = current_time + DAYINSEC;
+  a->got_app = 0;
+  a->state = 0;
+  a->txt = (char *)0;
+  (void)recur_apoint_check_next (a, current_time, get_today ());
+  (void)apoint_check_next (a, current_time);
+
+  return 1;
+}
+
+/*
+ * This is used for the daemon to check if we have an upcoming appointment or
+ * not.
+ */
+unsigned
+notify_get_next_bkgd (void)
+{
+  struct notify_app_s a;
+
+  a.txt = (char *)0;
+  if (!notify_get_next (&a))
+    return 0;
+
+  if (!a.got_app)
+    {
+      /* No next appointment, reset the previous notified one. */
+      notify_app.got_app = 0;
+      return 1;
+    }
+  else
+    {
+      if (!notify_same_item (a.time))
+        notify_update_app (a.time, a.state, a.txt);
+    }
+
+  if (a.txt)
+    mem_free (a.txt);
+  
+  return 1;
+}
+
+/* Return the description of next appointment to be notified. */
+char *
+notify_app_txt (void)
+{
+  if (notify_app.got_app)
+    return notify_app.txt;
+  else
+    return (char *)0;
+}
+
 /* Look for the next appointment within the next 24 hours. */
 /* ARGSUSED0 */
 static void *
 notify_thread_app (void *arg)
 {
   struct notify_app_s tmp_app;
-  time_t current_time;
 
-  current_time = time (NULL);
-
-  /* Use a temporary structure not to lock the mutex for a too
-   * long time while looking for next appointment. */
-  tmp_app.time = current_time + DAYINSEC;
-  tmp_app.got_app = 0;
-  tmp_app.txt = NULL;
-  tmp_app = *recur_apoint_check_next (&tmp_app, current_time, get_today ());
-  tmp_app = *apoint_check_next (&tmp_app, current_time);
-
-  pthread_mutex_lock (&notify_app.mutex);
-  if (tmp_app.got_app)
+  if (!notify_get_next (&tmp_app))
+    pthread_exit ((void *)0);
+  
+  if (!tmp_app.got_app)
     {
-      notify_update_app (tmp_app.time, tmp_app.state, tmp_app.txt);
+      pthread_mutex_lock (&notify_app.mutex);
+      notify_free_app ();
+      pthread_mutex_unlock (&notify_app.mutex);      
     }
   else
     {
-      notify_free_app ();
-      notify_app.got_app = 0;
-      notify_app.txt = 0;
+      if (!notify_same_item (tmp_app.time))
+        {
+          pthread_mutex_lock (&notify_app.mutex);          
+          notify_update_app (tmp_app.time, tmp_app.state, tmp_app.txt);
+          pthread_mutex_unlock (&notify_app.mutex);          
+        }
     }
-  pthread_mutex_unlock (&notify_app.mutex);
-
-  if (tmp_app.txt != NULL)
+  
+  if (tmp_app.txt)
     mem_free (tmp_app.txt);
   notify_update_bar ();
 
@@ -462,102 +569,140 @@ notify_start_main_thread (void)
   notify_check_next_app ();
 }
 
-/* Print options related to the notify-bar. */
+/*
+ * Print an option in the configuration menu.
+ * Specific treatment is needed depending on if the option is of type boolean
+ * (either YES or NO), or an option holding a string value.
+ */
 static void
-notify_print_options (WINDOW *optwin, int col)
+print_option (WINDOW *win, unsigned x, unsigned y, char *name,
+              char *valstr, unsigned valbool, char *desc, unsigned num)
 {
+  const int XOFF = 4;
+  const int MAXCOL = col - 3;
+  int x_opt, len;
+  
+  x_opt = x + XOFF + strlen (name);
+  mvwprintw (win, y, x, "[%u] %s", num, name);
+  erase_window_part (win, x_opt, y, MAXCOL, y);
+  if ((len = strlen (valstr)) != 0)
+    {
+      unsigned maxlen;
+
+      maxlen = MAXCOL - x_opt - 2;
+      custom_apply_attr (win, ATTR_HIGHEST);
+      if (len < maxlen)
+	mvwprintw (win, y, x_opt, "%s", valstr);
+      else
+	{
+          char buf[BUFSIZ];
+          
+	  (void)strncpy (buf, valstr, maxlen - 1);
+	  buf[maxlen - 1] = '\0';
+	  mvwprintw (win, y, x_opt, "%s...", buf);
+	}
+      custom_remove_attr (win, ATTR_HIGHEST);
+    }
+  else
+    print_bool_option_incolor (win, valbool, y, x_opt);
+  mvwprintw (win, y + 1, x, desc);
+}
+
+/* Print options related to the notify-bar. */
+static unsigned
+print_config_options (WINDOW *optwin)
+{
+  const int XORIG = 3;
+  const int YORIG = 0;
+  const int YOFF  = 3;
+  
   enum
-  { SHOW, DATE, CLOCK, WARN, CMD, NB_OPT };
+  { SHOW, DATE, CLOCK, WARN, CMD, DMON, DMON_LOG, NB_OPT };
 
   struct opt_s
   {
-    char name[BUFSIZ];
-    char desc[BUFSIZ];
-    char value[BUFSIZ];
-  }
-  opt[NB_OPT];
+    char     *name;
+    char     *desc;
+    char      valstr[BUFSIZ];
+    unsigned  valnum;
+  } opt[NB_OPT];
 
-  int i, y, x, l, x_pos, y_pos, x_offset, y_offset, maxcol, maxlen;
-  char buf[BUFSIZ];
+  int i;
 
-  x_pos = 3;
-  x_offset = 4;
-  y_pos = 4;
-  y_offset = 3;
-  maxcol = col - 2;
-
-  (void)strncpy (opt[SHOW].name, _("notify-bar_show = "), BUFSIZ);
-  (void)strncpy (opt[DATE].name, _("notify-bar_date = "), BUFSIZ);
-  (void)strncpy (opt[CLOCK].name, _("notify-bar_clock = "), BUFSIZ);
-  (void)strncpy (opt[WARN].name, _("notify-bar_warning = "), BUFSIZ);
-  (void)strncpy (opt[CMD].name, _("notify-bar_command = "), BUFSIZ);
-
-  (void)strncpy (opt[SHOW].desc,
-                 _("(if set to YES, notify-bar will be displayed)"), BUFSIZ);
-  (void)strncpy (opt[DATE].desc,
-                 _("(Format of the date to be displayed inside notify-bar)"),
-                 BUFSIZ);
-  (void)strncpy (opt[CLOCK].desc,
-                 _("(Format of the time to be displayed inside notify-bar)"),
-                 BUFSIZ);
-  (void)strncpy (opt[WARN].desc,
-                 _("(Warn user if an appointment is within next "
-                   "'notify-bar_warning' seconds)"), BUFSIZ);
-  (void)strncpy (opt[CMD].desc,
-                 _("(Command used to notify user of an upcoming appointment)"),
-                 BUFSIZ);
+  opt[SHOW].name = _("notify-bar_show = ");
+  opt[SHOW].desc = _("(if set to YES, notify-bar will be displayed)");
+  
+  opt[DATE].name = _("notify-bar_date = ");
+  opt[DATE].desc = _("(Format of the date to be displayed inside notify-bar)");
+  
+  opt[CLOCK].name = _("notify-bar_clock = ");
+  opt[CLOCK].desc = _("(Format of the time to be displayed inside notify-bar)");
+  
+  opt[WARN].name = _("notify-bar_warning = ");
+  opt[WARN].desc = _("(Warn user if an appointment is within next "
+                     "'notify-bar_warning' seconds)");
+  
+  opt[CMD].name = _("notify-bar_command = ");
+  opt[CMD].desc = _("(Command used to notify user of an upcoming appointment)");
+  
+  opt[DMON].name = _("notify-daemon_enable = ");
+  opt[DMON].desc = _("(Run in background to get notifications after exiting)");
+  
+  opt[DMON_LOG].name = _("notify-daemon_log = ");
+  opt[DMON_LOG].desc = _("(Log activity when running in background)");
 
   pthread_mutex_lock (&nbar.mutex);
 
-  (void)strncpy (opt[DATE].value, nbar.datefmt, BUFSIZ);
-  (void)strncpy (opt[CLOCK].value, nbar.timefmt, BUFSIZ);
-  (void)snprintf (opt[WARN].value, BUFSIZ, "%d", nbar.cntdwn);
-  (void)strncpy (opt[CMD].value, nbar.cmd, BUFSIZ);
+  /* String value options */
+  (void)strncpy (opt[DATE].valstr, nbar.datefmt, BUFSIZ);
+  (void)strncpy (opt[CLOCK].valstr, nbar.timefmt, BUFSIZ);
+  (void)snprintf (opt[WARN].valstr, BUFSIZ, "%d", nbar.cntdwn);
+  (void)strncpy (opt[CMD].valstr, nbar.cmd, BUFSIZ);
 
-  l = strlen (opt[SHOW].name);
-  x = x_pos + x_offset + l;
-  mvwprintw (optwin, y_pos, x_pos, "[1] %s", opt[SHOW].name);
-  erase_window_part (optwin, x, y_pos, maxcol, y_pos);
-  print_bool_option_incolor (optwin, nbar.show, y_pos, x);
-  mvwprintw (optwin, y_pos + 1, x_pos, opt[SHOW].desc);
+  /* Boolean options */
+  opt[SHOW].valnum = nbar.show;
+  pthread_mutex_unlock (&nbar.mutex);
+  
+  opt[DMON].valnum = dmon.enable;
+  opt[DMON_LOG].valnum = dmon.log;
 
-  for (i = 1; i < NB_OPT; i++)
+  opt[SHOW].valstr[0] = opt[DMON].valstr[0] = opt[DMON_LOG].valstr[0] = '\0';
+  
+  for (i = 0; i < NB_OPT; i++)
     {
-      l = strlen (opt[i].name);
-      y = y_pos + i * y_offset;
-      x = x_pos + x_offset + l;
-      maxlen = maxcol - x - 2;
-
-      mvwprintw (optwin, y, x_pos, "[%d] %s", i + 1, opt[i].name);
-      erase_window_part (optwin, x, y, maxcol, y);
-      custom_apply_attr (optwin, ATTR_HIGHEST);
-      if (strlen (opt[i].value) < maxlen)
-	mvwprintw (optwin, y, x, "%s", opt[i].value);
-      else
-	{
-	  (void)strncpy (buf, opt[i].value, maxlen - 1);
-	  buf[maxlen - 1] = '\0';
-	  mvwprintw (optwin, y, x, "%s...", buf);
-	}
-      custom_remove_attr (optwin, ATTR_HIGHEST);
-      mvwprintw (optwin, y + 1, x_pos, opt[i].desc);
+      int y;
+      
+      y = YORIG + i * YOFF;
+      print_option (optwin, XORIG, y, opt[i].name, opt[i].valstr,
+                    opt[i].valnum, opt[i].desc, i + 1);
     }
 
-  pthread_mutex_unlock (&nbar.mutex);
-  wmove (win[STA].p, 1, 0);
-  wnoutrefresh (optwin);
-  doupdate ();
+  return YORIG + NB_OPT * YOFF;
+}
+
+static void
+reinit_conf_win (scrollwin_t *win)
+{
+  unsigned first_line;
+
+  first_line = win->first_visible_line;
+  wins_scrollwin_delete (win);
+  custom_set_swsiz (win);  
+  wins_scrollwin_init (win);
+  wins_show (win->win.p, win->label);
+  win->first_visible_line = first_line;
 }
 
 /* Notify-bar configuration. */
 void
 notify_config_bar (void)
 {
-  window_t conf_win;
-  char label[BUFSIZ];
+  scrollwin_t cwin;
   char *buf;
   char *number_str =
-    _("Enter an option number to change its value [Q to quit] ");
+    _("Enter an option number to change its value");
+  char *keys =
+    _("(Press '^P' or '^N' to move up or down, 'Q' to quit)");
   char *date_str =
     _("Enter the date format (see 'man 3 strftime' for possible formats) ");
   char *time_str =
@@ -565,31 +710,43 @@ notify_config_bar (void)
   char *count_str =
     _("Enter the number of seconds (0 not to be warned before an appointment)");
   char *cmd_str = _("Enter the notification command ");
-  int ch = 0, change_win = 1;
+  int ch;
 
-  buf = mem_malloc (BUFSIZ);
-  (void)snprintf (label, BUFSIZ, _("notify-bar options"));
-  
-  conf_win.p = 0;
-  custom_confwin_init (&conf_win, label);
+  clear ();  
+  custom_set_swsiz (&cwin);
+  (void)snprintf (cwin.label, BUFSIZ, _("notification options"));
+  wins_scrollwin_init (&cwin);
+  wins_show (cwin.win.p, cwin.label);
+  status_mesg (number_str, keys);
+  cwin.total_lines = print_config_options (cwin.pad.p);
+  wins_scrollwin_display (&cwin);
 
-  while (ch != 'q')
+  buf = mem_malloc (BUFSIZ);  
+  while ((ch = wgetch (win[STA].p)) != 'q')  
     {
-      if (change_win)
-	custom_confwin_init (&conf_win, label);
-      status_mesg (number_str, "");
-      notify_print_options (conf_win.p, col);
-      *buf = '\0';
-      ch = wgetch (win[STA].p);
+      buf[0] = '\0';
 
       switch (ch)
-	{
-	case KEY_RESIZE:
-	  endwin ();
-	  refresh ();
-	  curs_set (0);
-	  delwin (conf_win.p);
-	  custom_confwin_init (&conf_win, label);
+        {
+        case KEY_RESIZE:
+          wins_get_config ();
+          wins_reset ();
+          reinit_conf_win (&cwin);
+	  delwin (win[STA].p);
+	  win[STA].p = newwin (win[STA].h, win[STA].w, win[STA].y,
+			       win[STA].x);
+	  keypad (win[STA].p, TRUE);
+	  if (notify_bar ())
+	    {
+	      notify_reinit_bar ();
+	      notify_update_bar ();
+	    }
+	  break;
+        case CTRL ('N'):
+          wins_scrollwin_down (&cwin, 1);
+	  break;
+        case CTRL ('P'):
+          wins_scrollwin_up (&cwin, 1);
 	  break;
 	case '1':
 	  pthread_mutex_lock (&nbar.mutex);
@@ -599,7 +756,8 @@ notify_config_bar (void)
 	    notify_start_main_thread ();
 	  else
 	    notify_stop_main_thread ();
-	  change_win = 1;
+          wins_scrollwin_delete (&cwin);
+          reinit_conf_win (&cwin);          
 	  break;
 	case '2':
 	  status_mesg (date_str, "");
@@ -612,7 +770,6 @@ notify_config_bar (void)
 	      (void)strncpy (nbar.datefmt, buf, strlen (buf) + 1);
 	      pthread_mutex_unlock (&nbar.mutex);
 	    }
-	  change_win = 0;
 	  break;
 	case '3':
 	  status_mesg (time_str, "");
@@ -625,7 +782,6 @@ notify_config_bar (void)
 	      (void)strncpy (nbar.timefmt, buf, strlen (buf) + 1);
 	      pthread_mutex_unlock (&nbar.mutex);
 	    }
-	  change_win = 0;
 	  break;
 	case '4':
 	  status_mesg (count_str, "");
@@ -639,7 +795,6 @@ notify_config_bar (void)
 	      nbar.cntdwn = atoi (buf);
 	      pthread_mutex_unlock (&nbar.mutex);
 	    }
-	  change_win = 0;
 	  break;
 	case '5':
 	  status_mesg (cmd_str, "");
@@ -652,10 +807,18 @@ notify_config_bar (void)
 	      (void)strncpy (nbar.cmd, buf, strlen (buf) + 1);
 	      pthread_mutex_unlock (&nbar.mutex);
 	    }
-	  change_win = 0;
 	  break;
-	}
+	case '6':
+	  dmon.enable = !dmon.enable;
+	  break;
+	case '7':
+	  dmon.log = !dmon.log;
+	  break;
+        }
+      status_mesg (number_str, keys);
+      cwin.total_lines = print_config_options (cwin.pad.p);
+      wins_scrollwin_display (&cwin);
     }
   mem_free (buf);
-  delwin (conf_win.p);
+  wins_scrollwin_delete (&cwin);
 }
