@@ -1,4 +1,4 @@
-/*	$calcurse: utils.c,v 1.76 2009-07-12 17:48:14 culot Exp $	*/
+/*	$calcurse: utils.c,v 1.82 2009-08-01 17:53:11 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -57,13 +57,14 @@
 #include "todo.h"
 #include "day.h"
 #include "keys.h"
+#include "dmon.h"
 #include "mem.h"
 
 /* General routine to exit calcurse properly. */
 void
 exit_calcurse (int status)
 {
-  int remove_lock;
+  int was_interactive;
   
   if (ui_mode == UI_CURSES)
     {
@@ -72,13 +73,31 @@ exit_calcurse (int status)
       refresh ();
       endwin ();
       ui_mode = UI_CMDLINE;
-      remove_lock = 1;
+      was_interactive = 1;
     }
   else
-    remove_lock = 0;
+    was_interactive = 0;
   
   calendar_stop_date_thread ();
   io_stop_psave_thread ();
+  free_user_data ();
+  keys_free ();
+  mem_stats ();
+  if (was_interactive)
+    {
+      if (unlink (path_cpid) != 0)
+        EXIT (_("Could not remove calcurse lock file: %s\n"),
+              strerror (errno));
+      if (dmon.enable)
+        dmon_start (status);
+    }
+  
+  exit (status);
+}
+
+void
+free_user_data (void)
+{
   day_free_list ();
   event_llist_free ();
   event_free_bkp (ERASE_FORCE);
@@ -90,12 +109,6 @@ exit_calcurse (int status)
   recur_event_free_bkp (ERASE_FORCE);
   todo_free_list ();
   notify_free_app ();
-  keys_free ();
-  mem_stats ();
-  if (remove_lock)
-    io_unset_lock ();
-  
-  exit (status);
 }
 
 /* Function to exit on internal error. */
@@ -761,6 +774,18 @@ now (void)
   return (current_time);
 }
 
+char *
+nowstr (void)
+{
+  static char buf[BUFSIZ];  
+  time_t now;
+
+  (void)time (&now);
+  (void)strftime (buf, sizeof buf, "%a %b %d %T %Y", localtime (&now));
+
+  return buf;
+}
+
 long
 mystrtol (const char *str)
 {
@@ -938,3 +963,17 @@ file_close (FILE *f, const char *pos)
   ret = fclose (f);
   EXIT_IF (ret != 0, _("Error when closing file at %s"), pos);
 }
+
+/*
+ * Sleep the given number of seconds, but make it more 'precise' than sleep(3)
+ * (hence the 'p') in a way that even if a signal is caught during the sleep
+ * process, this function will return to sleep afterwards.
+ */
+void
+psleep (unsigned secs)
+{
+  unsigned unslept;
+
+  for (unslept = sleep (secs); unslept; unslept = sleep (unslept))
+    ;
+}
