@@ -1,4 +1,4 @@
-/*	$calcurse: utils.h,v 1.46 2009-07-12 16:22:02 culot Exp $	*/
+/*	$calcurse: utils.h,v 1.49 2009-07-27 19:35:09 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -124,6 +124,7 @@ typedef enum
 erase_flag_e;
 
 void      exit_calcurse (int);
+void      free_user_data (void);
 void      fatalbox (const char *);
 void      warnbox (const char *);
 void      status_mesg (char *, char *);
@@ -149,6 +150,7 @@ void      draw_scrollbar (WINDOW *, int, int, int, int, int, unsigned);
 void      item_in_popup (char *, char *, char *, char *);
 long      get_today (void);
 long      now (void);
+char     *nowstr (void);
 long      mystrtol (const char *);
 void      print_bool_option_incolor (WINDOW *, unsigned, int, int);
 char     *new_tempfile (const char *, int);
@@ -156,5 +158,6 @@ void      erase_note (char **, erase_flag_e);
 int       parse_date (char *, int, int *, int *, int *);
 char     *str_toupper (char *);
 void      file_close (FILE *, const char *);
+void      psleep (unsigned);
 
 #endif /* CALCURSE_UTILS_H */
