@@ -1,4 +1,4 @@
-/*	$calcurse: vars.c,v 1.17 2009-07-12 16:22:02 culot Exp $	*/
+/*	$calcurse: vars.c,v 1.20 2009-08-01 13:31:21 culot Exp $	*/
 
 /*
  * Calcurse - text-based organizer
@@ -102,7 +102,9 @@ char path_apts[] = "";
 char path_conf[] = "";
 char path_notes[] = "";
 char path_keys[] = "";
-char path_lock[] = "";
+char path_cpid[] = "";
+char path_dpid[] = "";
+char path_dmon_log[] = "";
 
 /* Variable to handle pads. */
 struct pad_s apad;
@@ -110,6 +112,9 @@ struct pad_s apad;
 /* Variable to store notify-bar settings. */
 struct nbar_s nbar;
 
+/* Variable to store daemon configuration. */
+struct dmon_conf_s dmon;
+
 /*
  * Variables init 
  */
